clear;
close all;

%%%%%%%%%%%%%%%%%%%%%%
%%% fixed parameters
%%%%%%%%%%%%%%%%%%%%%%

% global r s lambda x alpha beta p z c sigma gamma n

p = 1;              % productivity
delta = .1;             % separation rate
r = .012;           % interest rate
z = .4;             % flow value of leisure
xx = 1.355;          % efficiency parameter in the matching function
alpha = .72;       % elasticity of the matching function
beta = alpha;      % bargaining power
c = .213;           % flow cost of posting a vacancy
sigma = .0165;      % standard deviation of prodctivity
gamma = .004;       % autoregressive parameter productivity
n = 1000;           % number of grid points = 2n+1

%%%%%%%%%%%%%%%%%%%%%%
%%% other parameters
%%%%%%%%%%%%%%%%%%%%%%

lambda = gamma*n;           %Poisson rate for productivity shocks
Delta = sigma/sqrt(lambda); %variation of log outpout after a shock occurs


Y = -cumsum(ones(n,1)).*Delta;
Y = [Y;cumsum(ones(n,1)).*Delta;0];
Y = sort(Y);
P = ones(2*n+1,1).*z +(p-z).*exp(Y);

T = zeros(2*n+1);   % T(i,j) gives transtition probabilities from state i to state j
T(1,2)=1;
T(end,end-1)=1;
N=2*n+1;
for i=2:N-1;
    i;
    T(i,i+1) = .5*(1-Y(i)/(n*Delta));
    T(i,i-1) = .5*(1+Y(i)/(n*Delta));
end;

Tcum=cumsum(T,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% simulating productivity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

periods=200;
s=n+1;
PPP=[P(s)];
time=0;
NB=[];              %number of shocks between two periods
for i=1:periods;
    nb=0;
    time=time-(log(1-rand)/lambda);
    while time<1;
    nb=nb+1;
    time=time-(log(1-rand)/lambda);
    end;
    NB=[NB;nb];
    time=0
end;

figure(1)
plot(NB),title('Number of shocks between each period')

S=[s];

for i=1:periods;
    if NB(i)==0
    PPP=[PPP;P(s)];
    S=[S;s];    
    else
        for k=1:NB(i);
            if s==1;
                s = 2;
            elseif s==2*n+1;
                s = 2*n;
            elseif rand < T(s,s-1);
                s = s-1;
            else
                s = s+1;
            end;
        end;
    PPP=[PPP;P(s)];
    S=[S;s];
    end;
end;


figure(2)
plot(PPP),title('Evolution of simulated productivity')


[trend,cycle] = hpfilter(log(PPP),1600);

CYCLE=exp(cycle);
TREND=exp(trend);

figure(3)
plot(CYCLE),title('Evolution of the cyclical component of simulated productivity')

figure(4)
plot(TREND),title('Evolution of the trend component of simulated productivity')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% hypothetical last period
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

THETA=[];

for i=1:2*n+1;
A = (r+delta+lambda)/xx;
B = beta;
C = (1-beta)*((P(i)-z)/c);
x = fsolve(@(x) last(x,A,B,C,alpha),(C/A)^(1/alpha),optimset('Display','off'));
THETA=[THETA;x];
end;

figure(10)
plot(P,THETA),title('last period decision'),xlabel('productivity'),ylabel('tightness \theta')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% convergence
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%expected search cost
E = lambda.*((THETA.^alpha)./xx);
E = T*E;

figure(11)
plot(P,E),ylabel('\lambda E\{1/q(\theta)\}'),xlabel('productivity')

%before the last tighness
THETAp=[];

for i=1:2*n+1;
A = (r+delta+lambda)/xx;
B = beta;
C = (1-beta)*((P(i)-z)/c)+E(i);
x = fsolve(@(x) last(x,A,B,C,alpha),(C/A)^(1/alpha),optimset('Display','off'));
THETAp=[THETAp;x];
end;

figure(12)
plot(THETA,THETAp),title('Comapring last tightness with before the last tightness'),ylabel('\theta_T'),xlabel('\theta_{T-1}')

figure(13)
plot(P,THETA,'b-',P,THETAp,'r-'),xlabel('productivity'),ylabel('tightness \theta'),legend('\theta_T','\theta_{T-1}')


%let's converge
figure(14)
plot(P,THETA,'b-',P,THETAp,'r-'),xlabel('productivity'),ylabel('tightness \theta')
hold on

D = 100

steps = 2

while D > .15;

THETA=THETAp;
E = lambda.*((THETA.^alpha)./xx);
E = T*E;
THETAp=[];

for i=1:2*n+1;
A = (r+delta+lambda)/xx;
B = beta;
C = (1-beta)*((P(i)-z)/c)+E(i);
x = fsolve(@(x) last(x,A,B,C,alpha),(C/A)^(1/alpha),optimset('Display','off'));
THETAp=[THETAp;x];
end;

plot(P,THETAp,'k-')

D = abs(THETA-THETAp)./THETA;
D = max(D);

steps = steps+1

end;

hold off



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Another simulation of productivity, tightness and unemployment
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% STDU=[];STDP=[];STDT=[];
% for kk=1:10000
% kk
    
    
periods=1212;
s=n+1;
u=.05;

PPP=[P(s)];
tightness=[THETAp(s)];
U=[u];

for i=1:periods;
    tau = -(log(1-rand)/lambda);
    time = tau;
    while time<1;
        if s==1;
            s = 2;
        elseif s==2*n+1;
            s = 2*n;
        elseif rand < T(s,s-1);
            s = s-1;
        else
            s = s+1;
        end;
        ppp = P(s);
        theta = THETAp(s);
        f = xx*(theta^(1-alpha));
        u = (delta/(delta+f))*(1-exp(-(delta+f)*tau))+u*exp(-(delta+f)*tau);
        tau = -(log(1-rand)/lambda);
        time = time + tau;
    end;
    PPP = [PPP ppp];
    U=[U u];
    tightness=[tightness theta];
end;

PPP=PPP(end-212:end);
U=U(end-212:end);
tightness=tightness(end-212:end);

figure(30)
plot(PPP),title('Simulated productivity'),xlabel('time')
figure(31)
plot(U),title('Simulated unemployment'),xlabel('time')
figure(32)
plot(tightness),title('Simulated v-u ratio'),xlabel('time')

PPP = log(PPP);
[trendP,cycleP]=hpfilter(PPP,100000);%cycleP=exp(cycleP);
U = log(U);
[trendP,cycleU]=hpfilter(U,100000);%cycleU=exp(cycleU);
tightness = log(tightness);
[trendP,cycle_tightness]=hpfilter(tightness,100000);%cycle_tightness=exp(cycle_tightness);

'Correlation between cyclical productivity, unemployment and tightness'
corr([cycleP cycleU cycle_tightness]);

'Standard deviation of cyclical productivity'
std(cycleP)
%STDP=[STDP std(cycleP)];

'Standard deviation of cyclical unemployment'
std(cycleU)
%STDU=[STDU std(cycleU)];

'Standard deviation of cyclical tighness'
std(cycle_tightness)
%STDT=[STDT std(cycle_tightness)];

% end;
% 
% 
% figure(40)
% hist(STDP),title('distribution of standard dev of productivity')
% figure(41)
% hist(STDT),title('distribution of standard dev of tightness')
% figure(42)
% hist(STDU),title('distribution of standard dev of unemployment')